"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ramda_1 = require("ramda");
var url = __importStar(require("url"));
var net_1 = require("../common/net");
var proxy_1 = require("../dsl/verifier/proxy");
var configurationError_1 = __importDefault(require("../errors/configurationError"));
var logger_1 = __importStar(require("../common/logger"));
var PactNative = __importStar(require("../../native/index.node"));
var VerifierV3 = /** @class */ (function () {
    function VerifierV3(config) {
        this.address = 'http://localhost';
        this.stateSetupPath = '/_pactSetup';
        this.config = config;
        this.validateConfiguration();
    }
    /**
     * Verify a HTTP Provider
     */
    VerifierV3.prototype.verifyProvider = function () {
        // Start the verification CLI proxy server
        var server = proxy_1.createProxy(this.config, this.stateSetupPath);
        // Run the verification once the proxy server is available
        // and properly shut down the proxy before returning
        return proxy_1.waitForServerReady(server)
            .then(this.runProviderVerification())
            .then(function (result) {
            return new Promise(function (resolve) {
                server.close(function () {
                    resolve(result);
                });
            });
        })
            .catch(function (e) {
            return new Promise(function (_, reject) {
                server.close(function () {
                    reject(e);
                });
            });
        });
    };
    VerifierV3.prototype.validateConfiguration = function () {
        var config = __assign({}, this.config);
        if (this.config.logLevel && !ramda_1.isEmpty(this.config.logLevel)) {
            logger_1.setLogLevel(this.config.logLevel);
        }
        if (this.config.validateSSL === undefined) {
            this.config.validateSSL = true;
        }
        if (this.config.changeOrigin === undefined) {
            this.config.changeOrigin = false;
            if (!this.isLocalVerification()) {
                this.config.changeOrigin = true;
                logger_1.default.warn("non-local provider address " + this.config.providerBaseUrl + " detected, setting 'changeOrigin' to 'true'. This property can be overridden.");
            }
        }
        if (ramda_1.isEmpty(this.config)) {
            throw new configurationError_1.default('no configuration provided to verifier');
        }
        // This is just too messy to do on the rust side. neon-serde would have helped, but appears unmaintained
        // and is currently incompatible
        if (this.config.consumerVersionSelectors) {
            config.consumerVersionSelectorsString = this.config.consumerVersionSelectors.map(function (s) { return JSON.stringify(s); });
        }
        if (!this.config.provider) {
            throw new configurationError_1.default('provider name is required');
        }
        if ((ramda_1.isEmpty(this.config.pactUrls) || !this.config.pactUrls) &&
            !this.config.pactBrokerUrl) {
            throw new configurationError_1.default('a list of pactUrls or a pactBrokerUrl must be provided');
        }
        return config;
    };
    // Run the Verification CLI process
    VerifierV3.prototype.runProviderVerification = function () {
        var _this = this;
        return function (server) {
            return new Promise(function (resolve, reject) {
                var opts = __assign(__assign({ providerStatesSetupUrl: _this.address + ":" + server.address().port + _this.stateSetupPath }, _this.config), { providerBaseUrl: _this.address + ":" + server.address().port });
                try {
                    PactNative.verify_provider(opts, function (err, val) {
                        if (err || !val) {
                            logger_1.default.trace({ err: err, val: val }, 'verification failed');
                            reject(err);
                        }
                        else {
                            logger_1.default.trace({ val: val }, 'verification succeeded');
                            resolve(val);
                        }
                    });
                    logger_1.default.trace('submitted test to verify_provider');
                }
                catch (e) {
                    reject(e);
                }
            });
        };
    };
    VerifierV3.prototype.isLocalVerification = function () {
        var u = new url.URL(this.config.providerBaseUrl);
        return (net_1.localAddresses.includes(u.host) || net_1.localAddresses.includes(u.hostname));
    };
    return VerifierV3;
}());
exports.VerifierV3 = VerifierV3;
//# sourceMappingURL=verifier.js.map